#ifndef _CHRONOPTICS_TOF_CAMERA_H_
#define _CHRONOPTICS_TOF_CAMERA_H_

#include <chronoptics/tof/stream.h>
#include <chronoptics/tof/processing_config.h>
#include <chronoptics/tof/data.h>
#include <chronoptics/tof/camera_config.h>
#include <chronoptics/tof/calibration.h>

#ifdef __cplusplus
extern "C" {
#endif

/** The main interface to the depth cameras
 */
typedef struct tof_camera* tof_camera_t;

/** Destruct tof_camera */
TOF_EXPORT void tof_camera_delete(tof_camera_t ptr);

/** Start streaming of the camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_start(tof_camera_t ptr, tof_error_t *error);

/** Stop streaming of the camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_stop(tof_camera_t ptr, tof_error_t *error);

/** Check whether the camera is streaming
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Is streaming
 */
TOF_EXPORT bool tof_camera_is_streaming(const tof_camera_t ptr, tof_error_t *error);

/** Check whether the camera is still connected
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Is connected
 */
TOF_EXPORT bool tof_camera_is_connected(const tof_camera_t ptr, tof_error_t *error);

/** Get the different output streams the camera can provide
 * @param ptr Pointer to class
 * @param streams The streams
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_get_stream_list(const tof_camera_t ptr, tof_stream_t* streams, size_t capacity, tof_error_t *error);

/** Set the streams that you want to get from the camera
 * @param ptr Pointer to class
 * @param streams The wanted streams
 * @param streams_size The size of the above array
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_set_stream_list(tof_camera_t ptr, const tof_stream_t* streams, size_t streams_size, tof_error_t *error);

/** Check whether the stream list is set
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Is stream list set
 */
TOF_EXPORT bool tof_camera_is_stream_list_set(const tof_camera_t ptr, tof_error_t *error);

/** Get the stream list that is set on the camera
 * @param ptr Pointer to class
 * @param streams The currently set streams
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_get_set_stream_list(tof_camera_t ptr, tof_stream_t* streams, size_t capacity, tof_error_t *error);

/** Get the processing config of the camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The processing config
 */
TOF_EXPORT tof_processing_config_t tof_camera_get_process_config(const tof_camera_t ptr, tof_error_t *error);

/** Set the processing config of the camera
 * @param ptr Pointer to class
 * @param config The processing config
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_set_process_config(tof_camera_t ptr, tof_processing_config_t config, tof_error_t *error);

/** Check if the camera has frames available
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Has frames available
 */
TOF_EXPORT bool tof_camera_has_frames(const tof_camera_t ptr, tof_error_t *error);

/** Get data frames from camera
 * @param ptr Pointer to class
 * @param frames Data frames
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_get_frames(tof_camera_t ptr, tof_data_t* frames, size_t capacity, tof_error_t *error);

/** Clear the circular buffer used to store frames
 * @param ptr Pointer to class
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_clear_buffer(tof_camera_t ptr, tof_error_t *error);

/** Get the circular buffer size
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The maximum number of sets of frames that is stored before the oldest
 * is overwritten
 */
TOF_EXPORT size_t tof_camera_get_buffer_size(const tof_camera_t ptr, tof_error_t *error);

/** Set the circular buffer size
 * @param ptr Pointer to class
 * @param size The maximum number of sets of frames that is stored before the
 * oldest is overwritten
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_set_buffer_size(tof_camera_t ptr, size_t size, tof_error_t *error);

/** Get the currently active camera config
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The camera config
 */
TOF_EXPORT tof_camera_config_t tof_camera_get_camera_config(const tof_camera_t ptr, tof_error_t *error);

/** Get the serial number of the current camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The serial
 */
TOF_EXPORT const char* tof_camera_get_serial(const tof_camera_t ptr, tof_error_t *error);

/** Get calibration from the camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The calibration
 */
TOF_EXPORT tof_calibration_t tof_camera_get_calibration(const tof_camera_t ptr, tof_error_t *error);

/** Get the amount of user pointers that can be stored
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The capacity
 */
TOF_EXPORT size_t tof_camera_get_user_pointer_capacity(const tof_camera_t ptr, tof_error_t *error);

/** Set the amount of user pointers that can be stored
 * @param ptr Pointer to class
 * @param capacity The capacity
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_set_user_pointer_capacity(tof_camera_t ptr, size_t capacity, tof_error_t *error);

/** Add a pointer that will be filled with the specified data type. This method
 * allows you to fill data straight into your own data type without any
 * additional copying.
 * @param ptr Pointer to class
 * @param pointer The pointer to the data to write into
 * @param capacity The amount of data the pointer can hold
 * @param callback Callback that will be called when pointer is no longer in use
 * @param callback_user_data User data that will be passed back when the function pointer is called
 * @param frame_type The frame type that will be filled into the pointer
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_add_user_pointer(tof_camera_t ptr, uint8_t* pointer, size_t capacity, tof_user_pointer_destructed_t callback, void* callback_user_data, enum tof_frame_type frame_type, tof_error_t *error);

/** Clear all user pointers in memory
 * @param ptr Pointer to class
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_clear_user_pointers(tof_camera_t ptr, tof_error_t *error);

/** Get the currently set configuration index
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return config index
 */
TOF_EXPORT uint32_t tof_camera_config_index(const tof_camera_t ptr, tof_error_t *error);

/** Switch to a different config in real time, depending on configuration and
 * camera, this can take 100-300ms
 * @param ptr Pointer to class
 * @param config_index Config index to switch to
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_switch_config(tof_camera_t ptr, uint32_t config_index, tof_error_t *error);

/** Get the frame types that this camera can output
 * @param ptr Pointer to class
 * @param frame_types Possible frame types
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_possible_frame_types(const tof_camera_t ptr, enum tof_frame_type* frame_types, size_t capacity, tof_error_t *error);

/** Whether output frame types have been set
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Output frame types set
 */
TOF_EXPORT bool tof_camera_output_frame_types_set(tof_camera_t ptr, tof_error_t *error);

/** Get the frame types the camera should output
 * @param ptr Pointer to class
 * @param output_frame_types Output frame types
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_get_output_frame_types(const tof_camera_t ptr, enum tof_frame_type* output_frame_types, size_t capacity, tof_error_t *error);

/** Set the frame types the camera should output
 * @param ptr Pointer to class
 * @param output_frame_types Output frame types
 * @param output_frame_types_size The size of the above array
 * @param error Pointer to error
 */
TOF_EXPORT void tof_camera_set_output_frame_types(tof_camera_t ptr, const enum tof_frame_type* output_frame_types, size_t output_frame_types_size, tof_error_t *error);

/** Check if multiple configurations are set. Switch config can only be used
 * when this is the case.
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Is set
 */
TOF_EXPORT bool tof_camera_has_multiple_configurations(const tof_camera_t ptr, tof_error_t *error);

/** Get all the set camera configurations
 * @param ptr Pointer to class
 * @param cam_configs Multiple camera configurations
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_get_multiple_camera_configurations(const tof_camera_t ptr, tof_camera_config_t* cam_configs, size_t capacity, tof_error_t *error);

/** Get all the set processing configurations
 * @param ptr Pointer to class
 * @param processing_configs Multiple processing configurations
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_get_multiple_processing_configurations(const tof_camera_t ptr, tof_processing_config_t* processing_configs, size_t capacity, tof_error_t *error);

/** Get the name for each configuration
 * @param ptr Pointer to class
 * @param names Names
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_get_multiple_names(const tof_camera_t ptr, const char** names, size_t capacity, tof_error_t *error);

/** Get the description for each configuration
 * @param ptr Pointer to class
 * @param description Descriptions
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_camera_get_multiple_descriptions(const tof_camera_t ptr, const char** description, size_t capacity, tof_error_t *error);

/** Select the streams you want to get from the camera
 * @param camera A camera
 * @param frame_types The frame types to select
 * @param frame_types_size The size of the above array
 * @param error Pointer to error
 * @return The amount of selected streams
 */
TOF_EXPORT size_t tof_select_streams(const tof_camera_t camera, const enum tof_frame_type* frame_types, size_t frame_types_size, tof_error_t *error);

/** Select the streams you want to get from the camera with the specified
 * modulation frequency
 * @param camera A camera
 * @param frame_types The frame types to select
 * @param frame_types_size The size of the above array
 * @param mod_freq The modulation frequency to select
 * @param error Pointer to error
 * @return The amount of selected streams
 */
TOF_EXPORT size_t tof_select_streams_mod_freq(const tof_camera_t camera, const enum tof_frame_type* frame_types, size_t frame_types_size, float mod_freq, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
